#!/usr/bin/env python3

#    Gedit a_href_picture plugin version 40.0.0
#    Copyright © 2011-2021 sasakima-nao <sasakimanao@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    2021.05.01 Version 40.0.0
#    get_uri_for_display to get_file
#
#    2019.09.12 Version 3.32.0
#    PyGtk compatible code removed
#    Delete code using deprecated stock
#    Unify string to single quote
#    GtkFileChooserDialog to GtkFileChooserNative
#
#    2018.05.03 Version 3.28.0
#    Support Gedit 3.28
#    'add_accelerator' to 'set_accels_for_action'
#
#    2016.06.23 Version 3.20.0
#    Support Gedit 3.20
#    remove '_' of action_name
#    'GObject.property' to 'GObject.Property'
#    append 'Plugin' in class name
#
#    2015.11.08 Version 3.18.0
#    Support Gedit 3.18
#    Add gi.require_version
#
#    2014.11.17 Version 3.12.0
#    Support Gedit 3.12
#    Keyboard ShortCut <Control><Alt>P
#
#    2014.01.03 Version 3.1.1
#    if not the Image then warning dialog
#    Under 300x300px is Not create subpicture
#    Fixed: Division '/' to '//'
#
#    2013.11.03 Version 3.1.0
#    Support Gedit 3.8 (Python3)
#
#    2011.07.18 Version 3.0.0

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gedit', '3.0')
gi.require_version('Peas', '1.0')

from gi.repository import GObject, Gedit, Gtk, Gio, GdkPixbuf
import os

MAX_SIZE = 300

class AHrefPictureAppActivatable(GObject.Object, Gedit.AppActivatable):
    '''
        Set GMenu and Accelerator
    '''
    app = GObject.Property(type=Gedit.App)
 
    def __init__(self):
        GObject.Object.__init__(self)
 
    def do_activate(self):
        self.app.set_accels_for_action('win.ahrefpicture', ['<Control><Alt>P'])
        self.menu_ext = self.extend_menu('file-section')
        item = Gio.MenuItem.new('a href Picture',  'win.ahrefpicture')
        self.menu_ext.append_menu_item(item)
 
    def do_deactivate(self):
        self.app.remove_accelerator('win.ahrefpicture', None)

class AHrefPicturePlugin(GObject.Object, Gedit.WindowActivatable):
    '''
        Create small image and link insert
        (e.g. <a href="img/pic1.jpg"><img src="img/pic1-300x225.jpg" alt="img/pic1.jpg" /></a>)
        This Plugin Gedit 3 only
    '''
    __gtype_name__ = 'AHrefPicturePlugin'
    window = GObject.Property(type=Gedit.Window)
    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        self.action = Gio.SimpleAction(name='ahrefpicture')
        self.action.connect('activate', self.on_a_href_activate)
        self.window.add_action(self.action)

    def do_deactivate(self):
        self.window.remove_action('ahrefpicture')

    def do_update_state(self):
        self.action.set_enabled(self.window.get_active_document() != None)

    def on_a_href_activate(self, action, param):
        doc = self.window.get_active_document()
        #viewpath = doc.get_uri_for_display()
        viewpath = doc.get_file().get_location().get_path()
        if viewpath[0] != '/':
            self.messagebox('Please save the file')
            return
        ft = Gtk.FileFilter()
        ft.add_pixbuf_formats()
        dlg = dlg = Gtk.FileChooserNative(
            filter=ft,
            title='Open',
            transient_for=self.window,
            action=Gtk.FileChooserAction.OPEN)
        dlg.set_current_folder(os.path.dirname(viewpath))
        r = dlg.run()
        if r == Gtk.ResponseType.ACCEPT:
            try:
                filepath = dlg.get_filename()
                pixbuf = GdkPixbuf.Pixbuf.new_from_file(filepath)
            except:
                dlg.destroy()
                self.messagebox('This File is Not Picure!')
                return
            # create under 300*300 px image
            aw = ah = MAX_SIZE
            #ah = MAX_SIZE
            w = pixbuf.get_width()
            h = pixbuf.get_height()
            cleate = False
            smallpath = ''
            if w > aw or h > ah:
                cleate = True
                if (aw * h) > (ah * w):
                    width = w * ah // h
                    height = ah
                else:
                    width = aw
                    height = h * aw // w
                smallpix = GdkPixbuf.Pixbuf.scale_simple(pixbuf, width, height, GdkPixbuf.InterpType.BILINEAR)
                # jpeg or png
                name, ext = os.path.splitext(filepath)
                smallpath = f'{name}-{width}x{height}{ext}'
                lext = ext.lower()
                if lext == '.jpg' or lext == '.jpeg':
                    smallpix.savev(smallpath, 'jpeg', ['quality'], ['85'])
                elif lext == '.png':
                    smallpix.savev(smallpath, 'png', ['compression'], ['9'])
                else:
                    self.messagebox(f'{lext} Extension not found')
                    return
            # create image link text
            dpath = os.path.dirname(viewpath)
            path1 = os.path.relpath(filepath, dpath)
            if cleate:
                path2 = os.path.relpath(smallpath, dpath)
                result = '<a href="{0}"><img src="{1}" alt="{0}" /></a>'.format(path1, path2)
            else:
                result = '<a href="{0}"><img src="{0}" alt="{0}" /></a>'.format(path1)
            # insert text
            view = self.window.get_active_view()
            buf = view.props.buffer.insert_at_cursor(result)
        dlg.destroy()

    def messagebox(self, text):
        dlg = Gtk.MessageDialog(
            buttons=Gtk.ButtonsType.OK,
            transient_for=self.window,
            use_markup=True,
            text=text)
        r = dlg.run()
        dlg.destroy()
