#!/usr/bin/env python3

#    Gedit html_escape plugin version 40.0.0
#    Copyright © 2011-2021 sasakima-nao <sasakimanao@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    2021.05.01 Version 40.0.0
#    Add Unescape Action
#
#    2019.09.12 Version 3.32.0
#    PyGtk compatible code removed
#    Unify string to single quote
#
#    2018.05.03 Version 3.28.0
#    Support Gedit 3.28
#    Change Keyboard ShortCut <Control><Alt>E
#    'add_accelerator' to 'set_accels_for_action'
#
#    2016.06.23 Version 3.20.0
#    Support Gedit 3.20
#    remove '_' of action_name
#    'GObject.property' to 'GObject.Property'
#    append 'Plugin' in class name
#
#    2015.11.08 Version 3.18.0
#    Support Gedit 3.18
#    Add gi.require_version
#
#    2014.11.17 Version 3.12.0
#    Support Gedit 3.12
#
#    2013.11.03 Version 3.1.0
#    Support Gedit 3.8 (Python3)
#
#    2011.06.04 Version 3.0.0

import gi, html
gi.require_version('Gtk', '3.0')
gi.require_version('Gedit', '3.0')
gi.require_version('Peas', '1.0')

from gi.repository import GObject, Gedit, Gtk, Gio

class HtmlEscapeAppActivatable(GObject.Object, Gedit.AppActivatable):
    '''
        Set GMenu and Accelerator
    '''
    app = GObject.Property(type=Gedit.App)
 
    def __init__(self):
        GObject.Object.__init__(self)
 
    def do_activate(self):
        self.app.set_accels_for_action('win.htmlescape', ['<Control><Alt>E'])
        self.app.set_accels_for_action('win.htmlunescape', ['<Control><Alt><Shift>E'])
        self.menu_ext = self.extend_menu('tools-section')
        item = Gio.MenuItem.new('HTML エスケイプ',  'win.htmlescape')
        self.menu_ext.append_menu_item(item)
        item2 = Gio.MenuItem.new('HTML アンエスケイプ',  'win.htmlunescape')
        self.menu_ext.append_menu_item(item2)
 
    def do_deactivate(self):
        self.app.remove_accelerator('win.htmlescape', None)
        self.app.remove_accelerator('win.htmlunescape', None)

class HtmlEscapePlugin(GObject.Object, Gedit.WindowActivatable):
    __gtype_name__ = 'HtmlEscapePlugin'
    window = GObject.Property(type=Gedit.Window)
    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        self.action = Gio.SimpleAction(name='htmlescape')
        self.action.connect('activate', self.on_htmlescape_activate)
        self.window.add_action(self.action)
        self.action2 = Gio.SimpleAction(name='htmlunescape')
        self.action2.connect('activate', self.on_htmlunescape_activate)
        self.window.add_action(self.action2)

    def do_deactivate(self):
        self.window.remove_action('htmlescape')
        self.window.remove_action('htmlunescape')

    def do_update_state(self):
        self.action.set_enabled(self.window.get_active_document() != None)
        self.action2.set_enabled(self.window.get_active_document() != None)

    def on_htmlescape_activate(self, action, param):
        view = self.window.get_active_view()
        buf = view.get_buffer()
        try:
            begin, end = buf.get_selection_bounds()
        except:
            self.messagebox('変換したいテキストを選択してください')
            return
        text = begin.get_text(end)
        result = html.escape(text, quote=False)
        buf.delete_selection(True, True)
        buf.insert_at_cursor(result)

    def on_htmlunescape_activate(self, action, param):
        view = self.window.get_active_view()
        buf = view.get_buffer()
        try:
            begin, end = buf.get_selection_bounds()
        except:
            self.messagebox('変換したいテキストを選択してください')
            return
        text = begin.get_text(end)
        result = html.unescape(text)
        buf.delete_selection(True, True)
        buf.insert_at_cursor(result)

    def messagebox(self, text):
        dlg = Gtk.MessageDialog(
            buttons=Gtk.ButtonsType.OK,
            transient_for=self.window,
            use_markup=True,
            text=text)
        r = dlg.run()
        dlg.destroy()
