#!/usr/bin/env python3

#    Gedit lf_to_br plugin version 3.32.0
#    Copyright © 2011-2019 sasakima-nao <sasakimanao@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    2019.09.12 Version 3.32.0
#    PyGtk compatible code removed
#    Unify string to single quote
#
#    2018.05.03 Version 3.28.0
#    Support Gedit 3.28
#    Change Keyboard ShortCut <Control><Alt>B
#    'add_accelerator' to 'set_accels_for_action'
#
#    2016.06.23 Version 3.20.0
#    Support Gedit 3.20
#    remove '_' of action_name
#    'GObject.property' to 'GObject.Property'
#    append 'Plugin' in class name
#
#    2015.11.08 Version 3.18.0
#    Support Gedit 3.18
#    Add gi.require_version
#
#    2014.11.17 Version 3.12.0
#    Support Gedit 3.12
#    Keyboard ShortCut <Control>B
#
#    2013.11.03 Version 3.1.0
#    Support Gedit 3.8 (Python3)
#    Support CRLF
#    Fix: GtkActionGroup Name
#
#    2011.06.04 Version 3.0.0

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gedit', '3.0')
gi.require_version('Peas', '1.0')

from gi.repository import GObject, Gedit, Gtk, Gio

class LF_on_BRAppActivatable(GObject.Object, Gedit.AppActivatable):
    '''
        Set GMenu and Accelerator
    '''
    app = GObject.Property(type=Gedit.App)
 
    def __init__(self):
        GObject.Object.__init__(self)
 
    def do_activate(self):
        self.app.set_accels_for_action('win.lfonbr', ['<Control><Alt>B'])
        self.menu_ext = self.extend_menu('tools-section')
        item = Gio.MenuItem.new('LF 位置に <br />',  'win.lfonbr')
        self.menu_ext.append_menu_item(item)
 
    def do_deactivate(self):
        self.app.remove_accelerator('win.lfonbr', None)

class LFonBRPlugin(GObject.Object, Gedit.WindowActivatable):
    __gtype_name__ = 'LFonBRPlugin'
    window = GObject.Property(type=Gedit.Window)
    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        self.action = Gio.SimpleAction(name='lfonbr')
        self.action.connect('activate', self.on_lfonbr_activate)
        self.window.add_action(self.action)

    def do_deactivate(self):
        self.window.remove_action('lfonbr')

    def do_update_state(self):
        self.action.set_enabled(self.window.get_active_document() != None)

    def on_lfonbr_activate(self, action, param):
        view = self.window.get_active_view()
        buf = view.get_buffer()
        try:
            begin, end = buf.get_selection_bounds()
        except:
            self.messagebox('変換したいテキストを選択してください')
            return
        text = begin.get_text(end)
        lf = '\n' if not  '\r' in text else '\r\n'
        lines = text.split(lf)
        result = []
        for line in lines:
            result.append(f'{line}<br />')
        buf.delete_selection(True, True)
        buf.insert_at_cursor(lf.join(result))

    def messagebox(self, text):
        dlg = Gtk.MessageDialog(
            buttons=Gtk.ButtonsType.OK,
            transient_for=self.window,
            use_markup=True,
            text=text)
        r = dlg.run()
        dlg.destroy()
