#!/usr/bin/env python3

#    Gedit repeat_line plugin version 40.0.0
#    Copyright © 2019-2021 sasakima-nao <sasakimanao@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; _endher version 2 of the License, or
#    (at your option) any later version.
#
#    2021.05.01 Version 40.0.0
#    fixed: Check cursol position is LF
#
#    2019.09.12 Version 3.32.0
#    PyGtk compatible code removed
#    Unify string to single quote
#    add <Alt><Super>Up
#    <Alt><Super>Down is move cursor
#
#    2018.05.03 Version 3.28.0
#    Support Gedit 3.28

import gi, os
gi.require_version('Gtk', '3.0')
gi.require_version('Gedit', '3.0')
gi.require_version('Peas', '1.0')

from gi.repository import GObject, Gedit, Gtk, Gio, GLib

class RepeatLineAppActivatable(GObject.Object, Gedit.AppActivatable):
    '''
        Set GMenu and Accelerator
    '''
    app = GObject.Property(type=Gedit.App)
 
    def __init__(self):
        GObject.Object.__init__(self)
 
    def do_activate(self):
        # '<Alt><Shift>Down' Not Work
        #self.app.set_accels_for_action('win.repeatup', ['<Alt><Super>Up'])
        #self.app.set_accels_for_action('win.repeatdown', ['<Alt><Super>Down'])
        self.app.set_accels_for_action('win.repeatup', ['<Control><Alt>u'])
        self.app.set_accels_for_action('win.repeatdown', ['<Control><Alt>d'])
 
    def do_deactivate(self):
        self.app.remove_accelerator('win.repeatup', None)
        self.app.remove_accelerator('win.repeatdown', None)

class RepeatLinePlugin(GObject.Object, Gedit.WindowActivatable):
    __gtype_name__ = 'RepeatLinePlugin'
    window = GObject.Property(type=Gedit.Window)
    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        self.action_up = Gio.SimpleAction(name='repeatup')
        self.action_down = Gio.SimpleAction(name='repeatdown')
        self.action_up.connect('activate', self.on_repeatup_activate)
        self.action_down.connect('activate', self.on_repeatdown_activate)
        self.window.add_action(self.action_up)
        self.window.add_action(self.action_down)

    def do_deactivate(self):
        self.window.remove_action('repeatup')
        self.window.remove_action('repeatdown')

    def do_update_state(self):
        self.action_up.set_enabled(self.window.get_active_document() != None)
        self.action_down.set_enabled(self.window.get_active_document() != None)

    def on_repeatup_activate(self, action, param):
        self.repeatline(False)

    def on_repeatdown_activate(self, action, param):
        self.repeatline(True)

    def repeatline(self, move):
        view = self.window.get_active_view()
        buf = view.get_buffer()
        top = buf.get_start_iter()
        buttom = buf.get_end_iter()
        start_pos = buf.props.cursor_position
        end_pos = start_pos
        it = buf.get_iter_at_offset(start_pos - 1)
        line = None
        # fix
        is_lf = buf.get_iter_at_offset(buf.props.cursor_position).get_char() == '\n'
        while 1:
            # search line start position
            if it.equal(top) or it.get_char() == '\n':
                line = it.copy()
                it = buf.get_iter_at_offset(end_pos)
                break
            start_pos -= 1
            it = buf.get_iter_at_offset(start_pos)
        while 1:
            # search line end position
            if it.equal(buttom) or it.get_char() == '\n':
                s = line.get_text(it)
                if line.equal(top):
                    s = '\n' + s
                buf.insert(it, s, -1)
                if move and not is_lf:
                    it_move = buf.get_iter_at_offset(buf.props.cursor_position)
                    it_move.set_offset(buf.props.cursor_position + len(s))
                    buf.place_cursor(it_move)
                break
            end_pos += 1
            it = buf.get_iter_at_offset(end_pos)

